/*
 * Copyright (c) 2010-2023. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.files;

import icy.file.Loader;
import icy.file.SequenceFileImporter;
import icy.plugin.abstract_.Plugin;
import icy.plugin.interface_.PluginBundled;
import icy.plugin.interface_.PluginLibrary;
import icy.roi.ROI;
import plugins.adufour.blocks.tools.io.IOBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.*;
import plugins.adufour.vars.util.VarException;
import plugins.tprovoost.sequenceblocks.SequenceBlocks;

import java.io.File;

/**
 * Block to returns a Sequence with the given parameters from the specified closed) SequenceFileImporter.
 *
 * @author Stephane
 */
public class LoadSubSequence extends Plugin implements IOBlock, PluginLibrary, PluginBundled {
    final protected Var<SequenceFileImporter> importer;
    final protected VarMutable file;
    final protected VarInteger series;
    final protected VarInteger resolution;
    final protected VarROIArray region;
    final protected VarInteger minZIndex;
    final protected VarInteger maxZIndex;
    final protected VarInteger minTIndex;
    final protected VarInteger maxTIndex;
    final protected VarInteger cIndex;
    final protected VarBoolean showProgress;

    final protected VarSequence sequence;

    public LoadSubSequence() {
        super();

        importer = new Var<>("Importer", SequenceFileImporter.class);
        file = new VarMutable("File", null) {
            @Override
            public boolean isAssignableFrom(final Var source) {
                return (String.class == source.getType()) || (File.class == source.getType());
            }
        };
        series = new VarInteger("Series", 0);
        resolution = new VarInteger("Resolution (0=full, 1=1/2, ..)", 0);
        region = new VarROIArray("XY region (ROI)", null);
        minZIndex = new VarInteger("Z min (slice)", -1);
        maxZIndex = new VarInteger("Z max (slice)", -1);
        minTIndex = new VarInteger("T min (frame)", -1);
        maxTIndex = new VarInteger("T max (frame)", -1);
        cIndex = new VarInteger("C (channel) index", -1);
        showProgress = new VarBoolean("Show progress", Boolean.FALSE);

        sequence = new VarSequence("Sequence", null);
    }

    @Override
    public void run() {
        final String path;
        final Object obj = file.getValue();

        if (obj != null) {
            if (obj instanceof String)
                path = (String) obj;
            else
                path = ((File) obj).getAbsolutePath();
        }
        else
            throw new VarException(file, "File is null !");

        // here importer can be null so we don't test for it
        final SequenceFileImporter imp = importer.getValue();
        final int s = series.getValue();
        final int r = resolution.getValue();
        final ROI[] rois = region.getValue();
        final ROI roi = ((rois != null) && (rois.length > 0)) ? rois[0] : null;
        final int minZ = minZIndex.getValue();
        final int maxZ = maxZIndex.getValue();
        final int minT = minTIndex.getValue();
        final int maxT = maxTIndex.getValue();
        final int c = cIndex.getValue();
        final boolean progress = showProgress.getValue();

        try {
            sequence.setValue(
                    Loader.loadSequence(
                            imp,
                            path,
                            s,
                            r,
                            (roi != null) ? roi.getBounds5D().toRectangle2D().getBounds() : null,
                            minZ,
                            maxZ,
                            minT,
                            maxT,
                            c,
                            false,
                            progress
                    )
            );
        }
        catch (final Exception e) {
            throw new VarException(importer, e.getMessage());
        }
    }

    @Override
    public void declareInput(final VarList inputMap) {
        inputMap.add("file", file);
        inputMap.add("importer", importer);
        inputMap.add("series", series);
        inputMap.add("resolution", resolution);
        inputMap.add("region", region);
        inputMap.add("minZ", minZIndex);
        inputMap.add("maxZ", maxZIndex);
        inputMap.add("minT", minTIndex);
        inputMap.add("maxT", maxTIndex);
        inputMap.add("cIndex", cIndex);
        inputMap.add("showProgress", showProgress);
    }

    @Override
    public void declareOutput(final VarList outputMap) {
        outputMap.add("sequence", sequence);
    }

    @Override
    public String getMainPluginClassName() {
        return SequenceBlocks.class.getName();
    }
}
