/*
 * Copyright (c) 2010-2023. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.files;

import icy.file.FileUtil;
import icy.file.ImageFileFormat;
import icy.file.Saver;
import icy.plugin.abstract_.Plugin;
import icy.plugin.interface_.PluginBundled;
import icy.plugin.interface_.PluginLibrary;
import icy.sequence.Sequence;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.*;
import plugins.adufour.vars.util.VarException;
import plugins.tprovoost.sequenceblocks.SequenceBlocks;

import java.io.File;

/**
 * Block to save a Sequence into a file
 *
 * @author Stephane
 */
public class SaveSequence extends Plugin implements SequenceBlock, PluginLibrary, PluginBundled {
    final protected VarMutable f_in = new VarMutable("File", null) {
        @Override
        public boolean isAssignableFrom(final Var source) {
            return String.class == source.getType() || File.class == source.getType();
        }
    };
    final protected VarBoolean overwriteForce = new VarBoolean("Overwrite", Boolean.TRUE);
    final protected VarBoolean multipleFile = new VarBoolean("Multiple file", Boolean.FALSE);
    final protected VarSequence sequence = new VarSequence("Sequence", null);
    final protected VarEnum<ImageFileFormat> format = new VarEnum<>("Format", ImageFileFormat.TIFF);

    @Override
    public void run() {
        final Sequence seq = sequence.getValue();

        if (seq != null) {
            final Object obj = f_in.getValue();

            if (obj != null) {
                String path;

                if (obj instanceof String)
                    path = (String) obj;
                else
                    path = ((File) obj).getAbsolutePath();

                if (FileUtil.isDirectory(path))
                    path += FileUtil.separator + FileUtil.getFileName(seq.getOutputFilename(false));

                // force extension depending file format
                path = FileUtil.setExtension(path, "." + format.getValue().getExtensions()[0]);

                // cannot overwrite ??
                if (FileUtil.exists(path) && !overwriteForce.getValue())
                    throw new VarException(f_in,
                            "File already exists. If you want to overwrite, please check the \"overwrite\" field.");

                // save sequence
                Saver.save(seq, new File(path), multipleFile.getValue(), false);
            }
        }
    }

    @Override
    public void declareInput(final VarList inputMap) {
        inputMap.add("file", f_in);
        inputMap.add("sequence", sequence);
        inputMap.add("format", format);
        inputMap.add("overwrite", overwriteForce);
        inputMap.add("multiple file", multipleFile);
    }

    @Override
    public void declareOutput(final VarList outputMap) {
        //
    }

    @Override
    public String getMainPluginClassName() {
        return SequenceBlocks.class.getName();
    }
}
