/*
 * Copyright (c) 2010-2023. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.infos;

import icy.plugin.abstract_.Plugin;
import icy.plugin.interface_.PluginBundled;
import icy.plugin.interface_.PluginLibrary;
import icy.sequence.Sequence;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.VarSequence;
import plugins.adufour.vars.lang.VarString;
import plugins.tprovoost.sequenceblocks.SequenceBlocks;

/**
 * Block to retrieve value (in String format) of a Sequence property (name, pixel size, custom...)<br>
 * Default metadata information can be retrieved using specific property name:<br>
 * <ul>
 * <li><i>id</i> = sequence internal Id</li>
 * <li><i>name</i> = sequence name</li>
 * <li><i>positionX</i> = absolute position X of the image (in um)</li>
 * <li><i>positionY</i> = absolute position Y of the image (in um)</li>
 * <li><i>positionZ</i> = absolute position Z of the image (in um)</li>
 * <li><i>positionT</i> = acquisition date time</li>
 * <li><i>positionTOffset</i> = not available for property;</li>
 * <li><i>pixelSizeX</i> = pixel size X (in um)</li>
 * <li><i>pixelSizeY</i> = pixel size Y (in um)</li>
 * <li><i>pixelSizeZ</i> = pixel size Z (in um)</li>
 * <li><i>timeInterval</i> = time interval between each image (in second)</li>
 * <li><i>channelName</i> = not available for property</li>
 * <li><i>userName</i> = user who generated / acquired the dataset</li>
 * <li><i>virtual</i> = virtual mode enabled on the dataset</li>
 * </ul>
 *
 * @author Stephane
 */
public class GetSequenceProperty extends Plugin implements SequenceBlock, PluginLibrary, PluginBundled {
    protected VarSequence sequence = new VarSequence("Sequence", null);
    protected VarString propertyName = new VarString("Property", "name");
    protected VarString value = new VarString("Value", "");

    @Override
    public void run() {
        final Sequence seq = sequence.getValue();

        if (seq != null)
            value.setValue(seq.getProperty(propertyName.getValue()));
    }

    @Override
    public void declareInput(final VarList inputMap) {
        inputMap.add("sequence", sequence);
        inputMap.add("property", propertyName);
    }

    @Override
    public void declareOutput(final VarList outputMap) {
        outputMap.add("value", value);
    }

    @Override
    public String getMainPluginClassName() {
        return SequenceBlocks.class.getName();
    }
}
