/*
 * Copyright (c) 2010-2023. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.infos;

import icy.math.UnitUtil;
import icy.plugin.abstract_.Plugin;
import icy.plugin.interface_.PluginBundled;
import icy.plugin.interface_.PluginLibrary;
import icy.sequence.MetaDataUtil;
import ome.xml.meta.OMEXMLMetadata;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarDouble;
import plugins.adufour.vars.lang.VarInteger;
import plugins.adufour.vars.lang.VarString;
import plugins.adufour.vars.util.VarException;
import plugins.tprovoost.sequenceblocks.SequenceBlocks;
import plugins.tprovoost.sequenceblocks.files.LoadMetadata;

/**
 * Block to return main informations from a metadata object.
 *
 * @author Stephane
 * @see LoadMetadata
 */
public class ReadMetadata extends Plugin implements SequenceBlock, PluginLibrary, PluginBundled {
    final protected Var<OMEXMLMetadata> metadata = new Var<>("Metadata", OMEXMLMetadata.class);
    final protected VarInteger serie = new VarInteger("Series", 0);

    final protected VarInteger numSerie = new VarInteger("Num serie", 1);
    final protected VarString name = new VarString("Name", "");
    final protected VarInteger sizeX = new VarInteger("Size X", 1);
    final protected VarInteger sizeY = new VarInteger("Size Y", 1);
    final protected VarInteger sizeC = new VarInteger("Size C", 1);
    final protected VarInteger sizeZ = new VarInteger("Size Z", 1);
    final protected VarInteger sizeT = new VarInteger("Size T", 1);
    final protected VarDouble pxSizeX = new VarDouble("Pixel Size X (" + UnitUtil.MICRO_STRING + "m)", 1d);
    final protected VarDouble pxSizeY = new VarDouble("Pixel Size Y (" + UnitUtil.MICRO_STRING + "m)", 1d);
    final protected VarDouble pxSizeZ = new VarDouble("Pixel Size Z (" + UnitUtil.MICRO_STRING + "m)", 1d);
    final protected VarDouble timeIntT = new VarDouble("Time interval T (s)", 100d);
    final protected VarDouble positionX = new VarDouble("Position X (" + UnitUtil.MICRO_STRING + "m)", 0d);
    final protected VarDouble positionY = new VarDouble("Position Y (" + UnitUtil.MICRO_STRING + "m)", 0d);
    final protected VarDouble positionZ = new VarDouble("Position Z (" + UnitUtil.MICRO_STRING + "m)", 0d);

    @Override
    public void run() {
        final OMEXMLMetadata meta = metadata.getValue();
        if (meta == null)
            throw new VarException(metadata, "Metadata is null !");

        final int s = serie.getValue();
        final int numS = MetaDataUtil.getNumSeries(meta);

        numSerie.setValue(numS);

        if (s >= numS)
            throw new VarException(serie, "Serie index must be between 0 and " + (numS - 1));

        name.setValue(MetaDataUtil.getName(meta, s));
        sizeX.setValue(MetaDataUtil.getSizeX(meta, s));
        sizeY.setValue(MetaDataUtil.getSizeY(meta, s));
        sizeZ.setValue(MetaDataUtil.getSizeZ(meta, s));
        sizeT.setValue(MetaDataUtil.getSizeT(meta, s));
        sizeC.setValue(MetaDataUtil.getSizeC(meta, s));
        pxSizeX.setValue(MetaDataUtil.getPixelSizeX(meta, s, pxSizeX.getDefaultValue()));
        pxSizeY.setValue(MetaDataUtil.getPixelSizeY(meta, s, pxSizeY.getDefaultValue()));
        pxSizeZ.setValue(MetaDataUtil.getPixelSizeZ(meta, s, pxSizeZ.getDefaultValue()));
        timeIntT.setValue(MetaDataUtil.getTimeInterval(meta, s, timeIntT.getDefaultValue()));
        // get position of first plane only for now
        positionX.setValue(MetaDataUtil.getPositionX(meta, s, 0, 0, 0, 0d));
        positionY.setValue(MetaDataUtil.getPositionY(meta, s, 0, 0, 0, 0d));
        positionZ.setValue(MetaDataUtil.getPositionZ(meta, s, 0, 0, 0, 0d));
    }

    @Override
    public void declareInput(final VarList inputMap) {
        inputMap.add("Metadata", metadata);
        inputMap.add("Serie", serie);
    }

    @Override
    public void declareOutput(final VarList outputMap) {
        outputMap.add("numSerie", numSerie);
        outputMap.add("Name", name);
        outputMap.add("Size X", sizeX);
        outputMap.add("Size Y", sizeY);
        outputMap.add("Size C", sizeC);
        outputMap.add("Size Z", sizeZ);
        outputMap.add("Size T", sizeT);
        outputMap.add("Pixel Size X (mm)", pxSizeX);
        outputMap.add("Pixel Size Y (mm)", pxSizeY);
        outputMap.add("Pixel Size Z (mm)", pxSizeZ);
        outputMap.add("Time interval T (ms)", timeIntT);
        outputMap.add("positionx", positionX);
        outputMap.add("positiony", positionY);
        outputMap.add("positionz", positionZ);
    }

    @Override
    public String getMainPluginClassName() {
        return SequenceBlocks.class.getName();
    }
}