/*
 * Copyright (c) 2010-2023. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.op;

import icy.plugin.abstract_.Plugin;
import icy.plugin.interface_.PluginBundled;
import icy.plugin.interface_.PluginLibrary;
import icy.roi.ROI;
import icy.roi.ROIUtil;
import icy.sequence.Sequence;
import icy.sequence.SequenceDataIterator;
import icy.type.DataIteratorUtil;
import icy.type.collection.CollectionUtil;
import icy.util.ShapeUtil.BooleanOperator;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.VarDouble;
import plugins.adufour.vars.lang.VarROIArray;
import plugins.adufour.vars.lang.VarSequence;
import plugins.adufour.vars.util.VarException;
import plugins.kernel.roi.roi3d.ROI3DBox;
import plugins.tprovoost.sequenceblocks.SequenceBlocks;

/**
 * Block to fill area outside the ROI regions with the specified value
 *
 * @author Stephane
 */
public class FillOuterSequence extends Plugin implements SequenceBlock, PluginLibrary, PluginBundled {
    final protected VarSequence inputSequence = new VarSequence("Sequence", null);
    final protected VarROIArray rois = new VarROIArray("Roi(s)");
    final protected VarDouble fillValue = new VarDouble("Value", 0d);

    @Override
    public void run() {
        final Sequence sequence = inputSequence.getValue();
        if (sequence == null)
            throw new VarException(inputSequence, "Input sequence is null.");

        try {
            if (rois.getValue() != null) {
                try {
                    final ROI roiUnion = ROIUtil.merge(CollectionUtil.asList(rois.getValue()), BooleanOperator.OR);
                    final ROI roiSeq = new ROI3DBox(sequence.getBounds5D().toRectangle3D());
                    final ROI roi = roiSeq.getSubtraction(roiUnion);

                    final double value = fillValue.getValue();

                    DataIteratorUtil.set(new SequenceDataIterator(sequence, roi), value);

                    sequence.dataChanged();
                }
                catch (final UnsupportedOperationException e) {
                    throw new VarException(rois, e.getMessage());
                }
            }
        }
        catch (final InterruptedException e) {
            // nothing to do
        }
    }

    @Override
    public void declareInput(final VarList inputMap) {
        inputMap.add("sequence", inputSequence);
        inputMap.add("rois", rois);
        inputMap.add("value", fillValue);
    }

    @Override
    public void declareOutput(final VarList outputMap) {
        //
    }

    @Override
    public String getMainPluginClassName() {
        return SequenceBlocks.class.getName();
    }
}